@@ -10,6 +10,7 @@ from jaseci.api.config_api import ConfigApi
 from jaseci.api.global_api import GlobalApi
 from jaseci.api.super_api import SuperApi
 from jaseci.api.actions_api import ActionsApi
+from jaseci.api.prometheus_api import PrometheusApi
 
 
 class SuperMaster(
@@ -20,6 +21,7 @@ class SuperMaster(
     GlobalApi,
     SuperApi,
     JsOrcApi,
+    PrometheusApi,
 ):
     """Master with admin APIs"""
 
