@@ -748,7 +748,7 @@ class Interp(VirtualMachine):
         atom:
             INT
             | FLOAT
-            | STRING
+            | multistring
             | BOOL
             | NULL
             | NAME
@@ -773,8 +773,8 @@ class Interp(VirtualMachine):
                 self.push(JacValue(self, value=int(kid[0].token_text())))
             elif kid[0].name == "FLOAT":
                 self.push(JacValue(self, value=float(kid[0].token_text())))
-            elif kid[0].name == "STRING":
-                self.push(JacValue(self, value=parse_str_token(kid[0].token_text())))
+            elif kid[0].name == "multistring":
+                self.run_multistring(kid[0])
             elif kid[0].name == "BOOL":
                 self.push(JacValue(self, value=bool(kid[0].token_text() == "true")))
             elif kid[0].name == "NULL":
@@ -1767,6 +1767,18 @@ class Interp(VirtualMachine):
         else:
             self.rt_error("Unrecognized type", kid[0])
 
+    def run_multistring(self, jac_ast):
+        """
+        multistring: STRING+;
+        """
+        if self.attempt_bytecode(jac_ast):
+            return
+        kid = self.set_cur_ast(jac_ast)
+        ret = ""
+        for i in kid:
+            ret += parse_str_token(i.token_text())
+        self.push(JacValue(self, value=ret))
+
     def destroy(self):
         """
         Destroys self from memory and persistent storage
@@ -1823,6 +1835,7 @@ class Interp(VirtualMachine):
                 "assignment",
                 "copy_assign",
                 "inc_assign",
+                "multistring",
             ]:
                 return self.pop()
             else:
