@@ -11,6 +11,9 @@ from jaseci.svc.actions_optimizer.actions_optimizer import ActionsOptimizer
 from .state import ServiceState as Ss
 from .config import META_CONFIG, KUBERNETES_CONFIG
 
+import time
+import numpy as np
+
 ###################################################
 #                  UNSAFE PARAMS                  #
 ###################################################
@@ -266,7 +269,18 @@ class JsOrc:
         self.services = {}
         self.background = {}
         self.context = {}
-        self.actions_optimizer = ActionsOptimizer()
+        self.benchmark = {
+            "jsorc": {"active": False, "requests": {}},
+            "actions_optimizer": {"active": False, "requests": {}},
+        }
+        self.actions_history = {"active": False, "history": []}
+        self.actions_calls = {}
+        self.system_states = {"active": False, "states": []}
+        self.actions_optimizer = ActionsOptimizer(
+            benchmark=self.benchmark["actions_optimizer"],
+            actions_history=self.actions_history,
+            actions_calls=self.actions_calls,
+        )
 
     ###################################################
     #                     BUILDER                     #
@@ -301,6 +315,8 @@ class JsOrc:
                 logger.exception(
                     f"Error checking {svc} !\n" f"{e.__class__.__name__}: {e}"
                 )
+        self.optimize(jsorc_interval=self.backoff_interval)
+        self.record_system_state()
 
     ###################################################
     #                   KUBERNETES                    #
@@ -322,18 +338,18 @@ class JsOrc:
         try:
             logger.info(f"Creating {kind} for `{name}` with namespace `{namespace}`")
             self.kubernetes.create(kind, namespace, conf)
-        except ApiException:
+        except ApiException as e:
             logger.error(
-                f"Error creating {kind} for `{name}` with namespace `{namespace}`"
+                f"Error creating {kind} for `{name}` with namespace `{namespace}` -- {e}"
             )
 
     def patch(self, kind: str, name: str, namespace: str, conf: dict):
         try:
             logger.info(f"Patching {kind} for `{name}` with namespace `{namespace}`")
             self.kubernetes.patch(kind, name, namespace, conf)
-        except ApiException:
+        except ApiException as e:
             logger.error(
-                f"Error patching {kind} for `{name}` with namespace `{namespace}`"
+                f"Error patching {kind} for `{name}` with namespace `{namespace}` -- {e}"
             )
 
     def read(self, kind: str, name: str, namespace: str):
@@ -342,7 +358,7 @@ class JsOrc:
             return self.kubernetes.read(kind, name, namespace=namespace)
         except ApiException as e:
             logger.error(
-                f"Error retrieving {kind} for `{name}` with namespace `{namespace}`"
+                f"Error retrieving {kind} for `{name}` with namespace `{namespace}` -- {e}"
             )
             return e
 
@@ -352,7 +368,7 @@ class JsOrc:
             return self.kubernetes.delete(kind, name, namespace=namespace)
         except ApiException as e:
             logger.error(
-                f"Error deleting {kind} for `{name}` with namespace `{namespace}`"
+                f"Error deleting {kind} for `{name}` with namespace `{namespace}` -- {e}"
             )
             return e
 
@@ -523,6 +539,173 @@ class JsOrc:
         """
         return self.actions_optimizer.get_actions_status(name)
 
+    def actions_tracking_start(self):
+        """ """
+        self.actions_history["active"] = True
+        self.actions_history["history"] = [{"ts": time.time()}]
+        self.actions_calls.clear()
+
+    def actions_tracking_stop(self):
+        """ """
+        if not self.actions_history["active"]:
+            return []
+
+        self.actions_optimizer.summarize_action_calls()
+
+        return self.actions_history["history"]
+
+    def benchmark_start(self):
+        """
+        Put JSORC under benchmark mode.
+        """
+        self.benchmark["jsorc"]["active"] = True
+        self.benchmark["jsorc"]["requests"] = {}
+        self.benchmark["jsorc"]["start_ts"] = time.time()
+
+    def state_tracking_start(self):
+        """
+        Ask JSORC to start tracking the state of the system as observed by JSORC on every interval.
+        """
+        self.system_states = {"active": True, "states": []}
+
+    def state_tracking_stop(self):
+        """
+        Stop state tracking for JSORC
+        """
+        ret = self.system_states
+        self.system_states = {"active": True, "states": []}
+        return ret
+
+    def state_tracking_report(self):
+        """
+        Return state tracking history so far
+        """
+        return self.system_states
+
+    def record_system_state(self):
+        """
+        Record system state
+        """
+        if self.system_states["active"]:
+            ts = int(time.time())
+            prom_profile = self.prometheus.info(
+                namespace=self.namespace,
+                exclude_prom=True,
+                timestamp=ts,
+                duration=self.backoff_interval,
+            )
+            self.system_states["states"].append(
+                {
+                    "ts": ts,
+                    "actions": self.get_actions_status(name=""),
+                    "prometheus": prom_profile,
+                }
+            )
+
+    def benchmark_stop(self, report):
+        """
+        Stop benchmark mode and report result during the benchmark period
+        """
+        if not self.benchmark["jsorc"]["active"]:
+            return {}
+
+        res = self.benchmark_report()
+        self.benchmark["jsorc"]["requests"] = {}
+        self.benchmark["jsorc"]["active"] = False
+
+        if report:
+            return res
+        else:
+            return {}
+
+    def benchmark_report(self):
+        """
+        Summarize benchmark results and report.
+        """
+        summary = {}
+        duration = time.time() - self.benchmark["jsorc"]["start_ts"]
+        for request, data in self.benchmark["jsorc"]["requests"].items():
+            summary[request] = {}
+            all_reqs = []
+            for req_name, times in data.items():
+                if len(times) == 0:
+                    continue
+                all_reqs.extend(times)
+                summary[request][req_name] = {
+                    "throughput": len(times) / duration,
+                    "average_latency": sum(times) / len(times) * 1000,
+                    "50th_latency": np.percentile(times, 50) * 1000,
+                    "90th_latency": np.percentile(times, 90) * 1000,
+                    "95th_latency": np.percentile(times, 95) * 1000,
+                    "99th_latency": np.percentile(times, 99) * 1000,
+                }
+            summary[request]["all"] = {
+                "throughput": len(all_reqs) / duration,
+                "average_latency": sum(all_reqs) / len(all_reqs) * 1000,
+                "50th_latency": np.percentile(all_reqs, 50) * 1000,
+                "90th_latency": np.percentile(all_reqs, 90) * 1000,
+                "95th_latency": np.percentile(all_reqs, 95) * 1000,
+                "99th_latency": np.percentile(all_reqs, 99) * 1000,
+            }
+
+        return summary
+
+    def record_state(self):
+        """
+        Record the current state of the system observed by JSORC
+        """
+
+    def add_to_benchmark(self, request_type, request, request_time):
+        """
+        Add requests to benchmark performance tracking
+        """
+        for bm in self.benchmark.values():
+            if request_type not in bm["requests"]:
+                bm["requests"][request_type] = {"_default_": []}
+            if request_type == "walker_run":
+                walker_name = dict(request.data)["name"]
+                if walker_name not in bm["requests"][request_type]:
+                    bm["requests"][request_type][walker_name] = []
+                bm["requests"][request_type][walker_name].append(request_time)
+            else:
+                bm["requests"][request_type]["_default_"].append(request_time)
+
+    def set_action_policy(self, policy_name, policy_params):
+        """
+        Set an action optimizer policy
+        """
+        return self.actions_optimizer.set_action_policy(policy_name, policy_params)
+
+    def get_action_policy(self):
+        """
+        Get the current action optimization policy
+        """
+        return self.actions_optimizer.get_action_policy()
+
+    def pre_action_call_hook(self, *args):
+        pass
+
+    def post_action_call_hook(self, *args):
+        action_name = args[0]
+        action_time = args[1]
+        if action_name not in self.actions_calls:
+            self.actions_calls[action_name] = []
+
+        self.actions_calls[action_name].append(action_time)
+
+    def pre_request_hook(self, *args):
+        pass
+
+    def post_request_hook(self, *args):
+        request_type = args[0]
+        request = args[1]
+        request_time = args[2]
+        if self.benchmark["jsorc"]["active"]:
+            self.add_to_benchmark(request_type, request, request_time)
+
+    def optimize(self, jsorc_interval):
+        self.actions_optimizer.run(jsorc_interval)
+
 
 class MetaProperties:
     def __init__(self, cls):
